class Site:
    def __init__(self, code: str,
                 id: int = None,
                 name: str = None,
                 remark: str = None,
                 gpsLatitude: float = None,
                 gpsLongitude: float = None,
                 altitude: float = None,
                 stationNetworkId = None,
                 client=None,
                 type="STATION",
                 **kwargs):
        self.id = id
        self.name = name
        self.remark = remark
        self.code = code
        self.gpsLatitude = gpsLatitude
        self.gpsLongitude = gpsLongitude
        self.altitude = altitude

        self._client = client

        self.stationNetworkId = stationNetworkId

        self.type = type

    @property
    def url(self):
        if self.code is not None:
            return "sites/" + self.code
        else:
            return "sites/id:" + str(self.id)

    @property
    def stationNetwork(self):
        return self._client._station_networks_manager.stationNetworkById(self.stationNetworkId)

    def data_sets(self):
        return self._client._data_sets_manager.data_sets(self.url, site=self)

    def data_set_by_code(self, element, aggregation, time_step, level=None, elementMarkOverride = None):
        return self._client._data_sets_manager.data_set_by_code(self, element, aggregation, time_step, level, elementMarkOverride)

    def delete_site(self):
        self._client._sites_manager.delete_site(self)



"""
.. include:: ./dataSet.md
"""
from datetime import datetime

import pandas as pd


class DataSets:

    def __init__(self, code: str, siteId: int = None, id: int = None, elementId: int = None, contextId: int = None, description: str = None,
                 elementMarkOverride=None, dataSetType: str = None, client=None, site=None):
        super().__init__()

        self.site = site
        self._client = client
        self.code = code

        self.elementId = elementId

        self.description = description
        self.element = self._client.element("id:"+str(self.elementId))
        self.id = id
        self.siteId = siteId
        self.contextId = contextId
        self.elementMarkOverride = elementMarkOverride
        self.dataSetType = dataSetType

    @property
    def url(self):
        return self.site.url + "/data-sets/" + self.code

    @property
    def flags(self):
        return self.element.flag_lists()

    @property
    def parameters(self):
        return self._client._parameters_manager.parameters(str(self.element.url) + "/parameters")

    def data(self, date_from: datetime = None, date_to: datetime = None):
        return self._client._data_sets_manager.get_data_set_data(self, date_from, date_to)

    def post_data(self, data: pd.DataFrame):
        return self._client._data_sets_manager.post_data_set_data2(data, self)

    def delete_data_set(self):
        return self._client._data_sets_manager.delete_data_set(self)

    def _mapping_flags_to_value(self):
        flags_lists = self.flags
        mapping = {}
        for flags_list in flags_lists:
            for flag in flags_list.flags:
                mapping[flag.id] = (flags_list.name, flag.code)
        return mapping

    def _mapping_parameters_to_col_name_type(self):
        parameters = self.parameters
        mapping = {}
        for parameter in parameters:
            mapping[str(parameter.id)] = (parameter.name, parameter.type)
        return mapping

    def _mapping_flags_values_to_id(self):
        flags_lists = self.flags
        mapping_lists = {}
        for flags_list in flags_lists:
            mapping_list = {}
            for flag in flags_list.flags:
                mapping_list[flag.code] = flag.id
            mapping_lists["flag-" + flags_list.name] = mapping_list
        return mapping_lists
    
    def _mapping_parameters_col_to_id_type(self):
        parameters = self.parameters
        mapping = {}
        for parameter in parameters:
            mapping["parameter-"+str(parameter.name)] = (parameter.id, parameter.type)
        return mapping



class FromToDateArgumentMissingException(Exception):
    def __init__(self, message="From/To date missing in arguments of function"):
        self.message = message
        super().__init__(self.message)





import json

from cgdb.resources.element import Element
from cgdb.utils.ManagerMix import ManagerMix


class ElementsManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def elements(self, url="elements"):
        content = self.get(url)
        elements = []

        for element_raw in content:
            elements.append(Element(**element_raw, client=self._client))

        return elements

    def element(self, mark):
        content = self.get("elements/" + mark)

        return Element(**content, client=self._client)

    def create_element(self, mark, description, defaultUnit, chartType, defaultColor="#ff0000", categoryId=2):
        out = {"categoryId": categoryId,
                "mark": mark,
                "description": description,
                "defaultUnit": defaultUnit,
                "defaultColor": defaultColor,
                "chartType": chartType}
        result = self.post("elements", json.dumps(out))
        if result.status_code != 200 and result.status_code != 204:
            raise Exception("upload fail on server side\n Server error:"+result.content.decode("ASCII"))

    def set_element_flags(self, element, list_of_flags_names):
        out = []
        for name in list_of_flags_names:
            out.append({"name": name})
        if isinstance(element, str):
            url_element  = "elements/" + element + "/flags"
        else:
            url_element = element.url + "/flags"
        result = self.post(url_element, json.dumps(out))
        if result.status_code != 200 and result.status_code != 204:
            raise Exception("upload fail on server side\n Server error:"+result.content.decode("ASCII"))

    # def add_element_flag(self, element, flag_list_name):
    #     self._client._flag_lists_manager.

    def set_element_parameters(self, element, list_of_parameters_names):
        out = []
        for name in list_of_parameters_names:
            out.append({"name": name})

        result = self.post(element.url+"/parameters", json.dumps(out))
        if result.status_code != 200:
            raise Exception("upload fail on server side\n Server error:"+result.content.decode("ASCII"))




from cgdb.resources.stationNetwork import StationNetwork
from cgdb.utils.ManagerMix import ManagerMix

class StationNetworksManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def stationNetworks(self):
        content = self.get("/station-networks")
        station_networks = []

        for station_network_raw in content:
            station_networks.append(StationNetwork(**station_network_raw, client=self._client))

        return station_networks

    def station_network_by_code(self, code):
        content = self.get("station-networks/" + code)

        return StationNetwork(**content, client=self._client)

    def station_network_by_id(self, id):
        content = self.get("station-networks/id:" + str(id))

        return StationNetwork(**content, client=self._client)


class ElementsCategory:
    def __init__(self, code: str, id: int = None, description: str = None, client=None):
        self.code = code
        self.id = id
        self.description = description

        self._client = client

    @property
    def url(self):
        return "/elements-categories/" + self.code

    def elements(self):
        self._client._elements_manager.elements(self.url + "/elements")



from cgdb.resources.elementsCategory import ElementsCategory
from cgdb.utils.ManagerMix import ManagerMix


class ElementsCategoriesManager(ManagerMix):
    def __init__(self, client):
        super().__init__(client)

    def elementsCategories(self):
        content = self.get("/element-categories")
        element_categories = []

        for element_category_raw in content:
            element_categories.append(ElementsCategory(**element_category_raw, client=self._client))

        return element_categories

    def elementsCategory(self, code):
        content = self.get("/element-categories/" + code)

        return ElementsCategory(**content, client=self._client)



import json

from cgdb.exceptions.EntityNotFound import EntityNotFoundException


class ManagerMix:
    def __init__(self, client):
        self._client = client

    def get(self, url=""):
        raw_content = self._client._session.get(url)

        if raw_content.status_code != 200:
            if raw_content.status_code == 404:
                content = json.loads(raw_content.content)
                raise EntityNotFoundException(content["code"], content["message"] + "\n Entity url: " + url)

        content = json.loads(raw_content.content)

        return content

    def post(self, url="", data=""):
        raw_content = self._client._session.post(url, data)
        print(raw_content.status_code)
        if raw_content.status_code != 200:
            if raw_content.status_code == 404:
                content = json.loads(raw_content.content)
                raise EntityNotFoundException(content["code"], content["message"] + "\n Entity url: " + url)

        return raw_content

    def delete(self, url=""):
        raw_content = self._client._session.delete(url)
        print(raw_content.status_code)
        if raw_content.status_code != 200:
            if raw_content.status_code == 404:
                content = json.loads(raw_content.content)
                raise EntityNotFoundException(content["code"], content["message"] + "\n Entity url: " + url)

        return raw_content



class TimeStep:
    def __init__(self, id: int, code: str, description: str, codePattern: str, timeStepInterval: dict, timeStepOffset, complex: bool):
        self.codePattern = codePattern
        self.timeStepInterval = timeStepInterval
        self.timeStepOffset = timeStepOffset
        self.complex = complex
        self.description = description
        self.id = id
        self.code = code

    @property
    def url(self):
        if self.code is not None:
            return "time-steps/" + self.code
        else:
            return "time-steps/id:" + str(self.id)



from cgdb.exceptions.ColumnNotFound import ColumnNotFoundException
from cgdb.exceptions.EntityNotFound import EntityNotFoundException
from cgdb.exceptions.FromToDateMissing import FromToDateArgumentMissingException


from cgdb.managers.aggregationsManager import AggregationsManager
from cgdb.managers.dataSetsManager import DataSetsManager
from cgdb.managers.departmentsManager import DepartmentsManager
from cgdb.managers.elementsManager import ElementsManager
from cgdb.managers.elementContextsManager import ElementContextsManager
from cgdb.managers.elementsCategoriesManager import ElementsCategoriesManager
from cgdb.managers.flagsListsManager import FlagListManager
from cgdb.managers.parametersManager import ParametersManager
from cgdb.managers.sitesManager import SitesManager
from cgdb.managers.stationNetworksManager import StationNetworksManager
from cgdb.managers.timeStepsManager import TimeStepsManager
from cgdb.managers.stationGroupsManager import StationGroupsManager
from cgdb.managers.levelsManager import LevelsManager



from cgdb.utils.ManagerMix import ManagerMix
from cgdb.resources.parameter import Parameter


class ParametersManager(ManagerMix):
    def __init__(self, client):
        self._client = client


    def parameters(self, path):
        content = self.get(path)
        parameters = []

        for parameters_raw in content:
            parameters.append(Parameter(**parameters_raw, client=self._client))

        return parameters



class Level:
    def __init__(self, id: int, code: str, description: str, level: float):
        self.level = level
        self.description = description
        self.id = id
        self.code = code

    @property
    def url(self):
        if self.code is not None:
            return "levels/" + self.code
        else:
            return "levels/id:" + str(self.id)



from setuptools import find_packages, setup

setup(
    name='cgdb_client',
    packages=find_packages(include=["cgdb", "cgdb.exceptions", "cgdb.managers", "cgdb.resources", "cgdb.utils"]),
    version='0.3.7',
    install_requires=["pandas==1.1.5", "requests==2.27.0"],
    description='CGDB Client',
    author='CzechGlobe',
    license='MIT',
)



class FlagList:
    def __init__(self, id:int, name:str=None, descripton:str=None, created:int=None, last_update:int=None, flags=None, client=None):
        self.id = id
        self.name = name
        self.descripton = descripton
        # self.created = created
        # self.last_update = last_update
        self.flags = []

        self.client = client
